__name__ = "anaconda_linter"
__version__ = "0.1.0"
__author__ = "Anaconda, Inc."
__email__ = "distribution_team@anaconda.com"
__license__ = "BSD-3-Clause"
__copyright__ = "Copyright 2022 Anaconda, Inc."
__summary__ = "A linter to validate recipe meta.yaml files."
__url__ = "https://github.com/anaconda-distribution/anaconda-linter"
